<?php

namespace Icinga\Module\Businessprocess\Monitoring\Backend\Ido\Query;

class ServiceStatusQuery extends \Icinga\Module\Monitoring\Backend\Ido\Query\ServicestatusQuery
{
    use CustomVarJoinTemplateOverride;
}
