<?php

namespace Icinga\Module\Businessprocess\Monitoring\Backend\Ido\Query;

class HostStatusQuery extends \Icinga\Module\Monitoring\Backend\Ido\Query\HoststatusQuery
{
    use CustomVarJoinTemplateOverride;
}
