<?= $this->controls->render() ?>

<div class="content">
<?php if ($this->showDiff): ?>
<div class="diff">
<?= $this->diff->render() ?>
</div>
<?php else: ?>
<table class="sourcecode">
<?php

$cnt = 0;
$lines = preg_split('~\r?\n~', $this->source);
$len = ceil(log(count($lines), 10));
$rowhtml = sprintf('<tr><th>%%0%dd: </th><td>%%s<br></td></tr>', $len);

foreach ($lines as $line) {
    $cnt++;
    printf($rowhtml, $cnt, $this->escape($line));
}

?>
</table>
<?php endif ?>
</div>
